/*
 * Rubik's clock.
 *
 * This file is public domain, I just hate copy[left/right]s.
 *-------------
 * the event handling stuff file
 */

#include "event.h"

#include <X11/Xlib.h>
#include <X11/keysym.h>

#include "clock.h"
#include "screen.h"

void wait_next_event(EVENT *ev, SCREEN *s)
{
  XEvent e;

  ev->type=-1;

/* I love gotos, sorry :) */
next_event:
  XNextEvent(s->d, &e);
  switch(e.type) {
    case Expose : {
      /* we gotta check window's size change (a simple GetGeometry is perfect) */
      Window wdum;
      unsigned int dum;
      XGetGeometry(s->d, s->w, &wdum, (int *)&dum, (int *)&dum, &s->sizex, &s->sizey, &dum, &dum);
      
      ev->type=REDRAW;
      break;
    }
    case MotionNotify : {
      XMotionEvent *e1=(XMotionEvent *)&e;
      ev->mousex=e1->x;
      ev->mousey=e1->y;
      ev->type=MOUSEMOVE;
      break;
    }
    case ButtonPress : {
      XButtonEvent *e1=(XButtonEvent *)&e;
      if ((ev->which=mouse_on_button(s, e1->x, e1->y))!=-1) {
	ev->type=BUTTON;
	break;
      }
      if ((ev->which=mouse_on_arrow(s, e1->x, e1->y))!=-1) {
	ev->type=CORNER;
	break;
      }
      if (mouse_on_turn(s, e1->x, e1->y)!=-1) {
	ev->type=TURN;
	break;
      }
      if (mouse_on_quit(s, e1->x, e1->y)!=-1) {
	ev->type=QUIT;
	break;
      }
      goto next_event;
    }
    case KeyPress : {
      int d;
      switch(d=XLookupKeysym((XKeyEvent *)&e, 0)) {
	case XK_space :
	case 't' :
	case 'T' :
	  ev->type=TURN;
	  return;
	case '4' :
	case 'y' :
	case 'Y' :
	  ev->type=BUTTON;
	  ev->which=0;
	  return;
	case 'u' :
	case 'U' :
	case '5' :
	  ev->type=BUTTON;
	  ev->which=1;
	  return;
	case 'h' :
	case 'H' :
	case '1' :
	  ev->type=BUTTON;
	  ev->which=2;
	  return;
	case 'j' :
	case 'J' :
	case '2' :
	  ev->type=BUTTON;
	  ev->which=3;
	  return;
	case XK_Escape :
	case 'q' :
	case 'Q' :
	  ev->type=QUIT;
	  return;
	case 'b' :
	case 'B' :
	case XK_Left : {
	  int which[]={4, 6, 2, 0};
	  ev->type=CORNER;
	  ev->which=which[s->current_corner];
	  return;
	}
	case 'n' :
	case 'N' :
	case XK_Right : {
 	  int which[]={5, 7, 3, 1};
	  ev->type=CORNER;
	  ev->which=which[s->current_corner];
	  return;
	}
	case XK_Up : {
	  int next[]={1, 3, 0, 2};
	  ev->which=next[s->current_corner];
	  ev->type=SELECT_CORNER;
	  return;
	}
	case XK_Down : {
	  int next[]={2, 0, 3, 1};
	  ev->which=next[s->current_corner];
	  ev->type=SELECT_CORNER;
	  return;
	}
	case 'e' :
	case 'E' :
	  ev->which=0;
	  ev->type=SELECT_CORNER;
	  return;
	case 'r' :
	case 'R' :
	  ev->which=1;
	  ev->type=SELECT_CORNER;
	  return;
	case 'd' :
	case 'D' :
	  ev->which=2;
	  ev->type=SELECT_CORNER;
	  return;
	case 'f' :
	case 'F' :
	  ev->which=3;
	  ev->type=SELECT_CORNER;
	  return;
	/* some hack functions */
	case XK_F2 :
	  ev->type=F2;
	  return;
	case XK_F3 :
	  ev->type=F3;
	  return;
      }
      goto next_event;
    }
  }
}

void handle_event(EVENT *ev, CLOCK *c, SCREEN *s)
{
  switch(ev->type) {
    case REDRAW :
      draw_all(c, s);
      break;
    case MOUSEMOVE :
      draw_mouse(c, s, ev->mousex, ev->mousey);
      break;
    case TURN :
      return_clock(c, s);
      break;
    case BUTTON :
      push_button(c, ev->which, s);
      break;
    case CORNER : {
      int corn[]={3, 3, 2, 2, 0, 0, 1, 1};
      turn_corner(c, corn[ev->which], ev->which & 1, s);
      break;
    }
    case SELECT_CORNER :
      select_corner(s, ev->which);
      break;
    case F2 :
      init_clock(c, s);
      draw_all(c, s);
      break;
    case F3 :
      randomize_clock(c);
      draw_all(c, s);
      break;
  }
}
